<?php

/**
 * @file
 * The user tab page callbacks and other related functions.
 */

/**
 * Page callback for the memberships tab page.
 */
function membersify_account_tab($account) {
  $memberships = membersify_get_memberships($account->uid);
  $membership_html_array = array();
  $plans = membersify_get_plans();
  foreach ($memberships as $membership) {
    $cancel_url = NULL;
    $change_url = NULL;
    $billing_url = NULL;
    $renew_url = NULL;
    $history_url = url('user/' . $account->uid . '/membership/' . $membership->id . '/history');
    $plan = membersify_plan_load($membership->plan_id);
    $modify_options = membersify_get_modify_options($plan);

    if (in_array($membership->status, array('active', 'pending_downgrade', 'expiring_soon'))) {
      $billing_url = url('user/' . $account->uid . '/membership/' . $membership->id . '/change_billing');
    }
    if (!empty($modify_options) && in_array($membership->status, array('active', 'expiring_soon'))) {
      $change_url = url('user/' . $account->uid . '/membership/' . $membership->id . '/change_plan');
    }
    if (in_array($membership->status, array('active'))) {
      $cancel_url = url('user/' . $account->uid . '/membership/' . $membership->id . '/cancel');
    }
    if (in_array($membership->status, array('canceled'))) {
      $renew_url = url('user/' . $account->uid . '/membership/' . $membership->id . '/renew');
    }

    $membership_html_array[] = theme('membersify_membership', array(
      'membership' => $membership,
      'plan' => $plans[$membership->plan_id],
      'change_billing_url' => $billing_url,
      'change_plan_url' => $change_url,
      'cancel_url' => $cancel_url,
      'renew_url' => $renew_url,
      'history_url' => $history_url,
    ));
  }

  $purchase_links_array = array();
  foreach ($plans as $plan) {
    $purchase_links_array[] = theme('membersify_purchase_link', array(
      'plan' => $plan,
      'url' => url('membersify/purchase/' . $plan->id),
    ));
  }

  drupal_add_css(drupal_get_path('module', 'membersify') . '/css/membersify.user.css');

  return theme('membersify_user_page', array(
    'account' => $account,
    'memberships_html' => implode('', $membership_html_array),
    'purchase_links' => implode('', $purchase_links_array),
  ));
}

/**
 * Form constructor for the cancel membership form.
 */
function membersify_cancel_membership_form($form, &$form_state, $account, Membersify_Membership $membership) {
  $form['question'] = array(
    '#markup' => t("Are you sure you want to cancel this membership?"),
  );
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Yes, cancel my membership'),
  );
  $form['actions']['cancel'] = array(
    '#markup' => l(t('Wait, I changed my mind'), 'user/' . $account->uid . '/membership'),
  );

  return $form;
}

/**
 * Form validation handler for membersify_cancel_membership_form().
 */
function membersify_cancel_membership_form_validate($form, &$form_state) {
  try {
    $membership = $form_state['build_info']['args'][1];
    Membersify::setKeys(variable_get('membersify_public_key', ''), variable_get('membersify_secret_key', ''));
    $membership->cancel();
  }
  catch (Exception $e) {
    form_set_error('submit', $e->getMessage());
  }
}

/**
 * Form submission handler for membersify_cancel_membership_form().
 */
function membersify_cancel_membership_form_submit($form, &$form_state) {
  drupal_set_message("Your membership has been cancelled.");
  $account = $form_state['build_info']['args'][0];
  $form_state['redirect'] = 'user/' . $account->uid . '/membership';
}

/**
 * Form constructor for the renew membership form.
 *
 * @see membersify_renew_membership_form_validate()
 * @see membersify_renew_membership_form_submit()
 * @see membersify_menu()
 */
function membersify_renew_membership_form($form, &$form_state, $account, Membersify_Membership $membership) {
  $form['question'] = array(
    '#markup' => t("Are you sure you want to renew this membership?"),
  );
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Yes, renew my membership'),
  );
  $form['actions']['cancel'] = array(
    '#markup' => l(t('Wait, I changed my mind'), 'user/' . $account->uid . '/membership'),
  );

  return $form;
}

/**
 * Form validation handler for membersify_renew_membership_form().
 */
function membersify_renew_membership_form_validate($form, &$form_state) {
  try {
    $membership = $form_state['build_info']['args'][1];
    Membersify::setKeys(variable_get('membersify_public_key', ''), variable_get('membersify_secret_key', ''));
    $result = $membership->renew();
    membersify_save_record('membersify_memberships', $result);
  }
  catch (Exception $e) {
    form_set_error('submit', $e->getMessage());
  }
}

/**
 * Form submission handler for membersify_renew_membership_form().
 */
function membersify_renew_membership_form_submit($form, &$form_state) {
  drupal_set_message("Your membership has been renewed.");
  $account = $form_state['build_info']['args'][0];
  $form_state['redirect'] = 'user/' . $account->uid . '/membership';
}

/**
 * Form constructor for the change plan form.
 */
function membersify_change_plan_form($form, &$form_state, $account, Membersify_Membership $membership) {
  $current_plan = membersify_plan_load($membership->plan_id);

  $form['plan_id'] = array(
    '#type' => 'radios',
    '#title' => t('New membership level'),
    '#description' => t('Choose your new membership level'),
    '#options' => membersify_get_modify_options($current_plan),
    '#required' => TRUE,
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Modify'),
  );
  $form['actions']['cancel'] = array(
    '#markup' => l(t('Wait, I changed my mind.'), 'user/' . $account->uid . '/membership'),
  );

  return $form;
}

/**
 * Form validation handler for membersify_change_plan_form().
 */
function membersify_change_plan_form_validate($form, &$form_state) {
  try {
    $membership = $form_state['build_info']['args'][1];
    Membersify::setKeys(variable_get('membersify_public_key', ''), variable_get('membersify_secret_key', ''));
    $membership->modify($form_state['values']['plan_id']);
  }
  catch (Exception $e) {
    form_set_error('submit', $e->getMessage());
  }
}

/**
 * Form submission handler for membersify_change_plan_form().
 */
function membersify_change_plan_form_submit($form, &$form_state) {
  drupal_set_message("Your membership has been changed.");
  $account = $form_state['build_info']['args'][0];
  $form_state['redirect'] = 'user/' . $account->uid . '/membership';
}

/**
 * Form constructor for the change billing form.
 *
 * @param stdClass $account
 *   The user account.
 * @param Membersify_Membership $membership
 *   The membership object.
 *
 * @see membersify_change_billing_form_validate()
 * @see membersify_change_billing_form_submit()
 * @ingroup forms
 */
function membersify_change_billing_form($form, &$form_state, $account, Membersify_Membership $membership) {
  $form['#id'] = 'membersify-add-card-button-form';
  // Also show a button to add a new payment profile.
  // Include the js from stripe.com.
  $form['#attached']['js'][] = array(
    'data' => 'https://checkout.stripe.com/checkout.js',
    'type' => 'external'
  );
  $form['#attached']['js'][] = array(
    'data' => array('stripe' => array(
      'publicKey' => variable_get('membersify_stripe_account', ''),
      'label' => t("Add new card"),
      'email' => $account->mail,
    )),
    'type' => 'setting'
  );
  $form['#attached']['js'][] = drupal_get_path('module', 'membersify') . '/js/membersify.stripe.js';
  $form['#attached']['css'][] = drupal_get_path('module', 'membersify') . '/css/membersify.stripe.css';
  $form['#attached']['css'][] = drupal_get_path('module', 'membersify') . '/css/membersify.user.css';

  $form['token_field'] = array(
    '#type' => 'hidden',
    '#attributes' => array(
      'class' => array('membersify_add_card_token_field'),
    ),
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['add_card_button'] = array(
    '#markup' => "<a id='membersify_add_card_button' href='#' class='stripe-connect'><span>" . t('Add new card') . "</span></a>"
  );

  $form['actions']['cancel'] = array(
    '#markup' => l(t('Back to memberships overview'), 'user/' . $account->uid . '/membership'),
  );

  // We need a button or else jQuery submit won't work. But it will be hidden.
  $form['submit'] = array(
    '#type' => 'submit',
    '#prefix' => "<div id='membersify-billing-submit-button-hidden'>",
    '#suffix' => '</div>',
    '#value' => t('Pay with card'),
  );

  $profiles = membersify_get_payment_profiles($account->uid);
  $form['payment_profiles'] = array(
    '#type' => 'fieldset',
    '#title' => t("Cards"),
    '#prefix' => "<div class='membersify-profiles'>",
    '#suffix' => "</div>",
  );

  foreach ($profiles as $profile) {
    $classes = array('membersify-profile', 'membersify-profile-' . $profile->status);
    $actions = array();
    $statuses = array(membersify_get_payment_profile_status($profile->status));
    if ($profile->status == 'active' && $profile->id != $membership->payment_profile_id) {
      $actions[] = l(t('Make default'), 'user/' . $account->uid . '/membership/' . $membership->id . '/change_billing/' . $profile->id . '/set_default',
        array('attributes' => array('class' => array('membersify_button'))));
    }
    $memberships = membersify_get_memberships_by_payment_profile_id($profile->id);
    if (empty($memberships)) {
      $actions[] = l(t('Delete'), 'user/' . $account->uid . '/membership/' . $membership->id . '/change_billing/' . $profile->id . '/delete',
          array('attributes' => array('class' => array('membersify_button'))));
    }
    else {
      if ($profile->id == $membership->payment_profile_id) {
        $statuses[] =t("Default");
        $classes[] = 'membersify-profile-default';
      }
      else {
        $statuses[] =t("In use");
        $classes[] = 'membersify-profile-in-use';
      }
    }



    $form['payment_profiles'][$profile->id] = array(
      '#type' => 'markup',
      '#prefix' => "<div class='" . implode(' ', $classes) ."'>",
      '#suffix' => "<div class='membersify-clearfix'></div></div>",
      '#markup' => "<div class='membersify-profile-display'>" . $profile->display . "</div><div class='membersify-profile-status'>" . implode(', ', $statuses) . "</div><div class='membersify-profile-actions'>" . implode('', $actions)  . "</div>",
    );
  }

  return $form;
}

/**
 * Form validation handler for membersify_change_billing_form().
 */
function membersify_change_billing_form_validate($form, &$form_state) {
  try {
    Membersify::setKeys(variable_get('membersify_public_key', ''), variable_get('membersify_secret_key', ''));
    $account = $form_state['build_info']['args'][0];
    // If the user is adding a new card, let's save it.
    if (!empty($form_state['values']['token_field'])) {
      $params['token'] = $form_state['values']['token_field'];
      $params['user_id'] = $account->uid;

      $payment_profile = Membersify_Profile::create($params);

      // Save the payment profile.
      membersify_save_record('membersify_payment_profiles', $payment_profile);
    }
  }
  catch (Exception $e) {
    form_set_error('payment_profile_id', $e->getMessage());
  }
}

/**
 * Form submission handler for membersify_change_billing_form().
 */
function membersify_change_billing_form_submit($form, &$form_state) {
  drupal_set_message(t("Your card has been added successfully."));
}

/**
 * Form constructor for the set default profile form.
 *
 * @see membersify_set_default_profile_form_validate()
 * @see membersify_set_default_profile_form_submit()
 * @see membersify_menu()
 */
function membersify_set_default_profile_form($form, &$form_state, $account, Membersify_Membership $membership, Membersify_Profile $profile) {
  $form['question'] = array(
    '#markup' => t("Are you sure you want to set your default card to %display?", array('%display' => $profile->display)),
  );
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Yes, set default'),
  );
  $form['actions']['cancel'] = array(
    '#markup' => l(t('Wait, I changed my mind'), 'user/' . $account->uid . '/membership/' . $membership->id . '/change_billing'),
  );

  return $form;
}

/**
 * Form validation handler for membersify_set_default_profile_form().
 */
function membersify_set_default_profile_form_validate($form, &$form_state) {
  try {
    $membership = $form_state['build_info']['args'][1];
    $profile = $form_state['build_info']['args'][2];
    Membersify::setKeys(variable_get('membersify_public_key', ''), variable_get('membersify_secret_key', ''));
    $membership->changeBilling($profile->id);
  }
  catch (Exception $e) {
    form_set_error('submit', $e->getMessage());
  }
}

/**
 * Form submission handler for membersify_set_default_profile_form().
 */
function membersify_set_default_profile_form_submit($form, &$form_state) {
  drupal_set_message("The default card has been updated.");
  $account = $form_state['build_info']['args'][0];
  $membership = $form_state['build_info']['args'][1];
  $form_state['redirect'] = 'user/' . $account->uid . '/membership/' . $membership->id . '/change_billing';
}

/**
 * Form constructor for the delete profile form.
 *
 * @see membersify_delete_profile_form_validate()
 * @see membersify_delete_profile_form_submit()
 * @see membersify_menu()
 */
function membersify_delete_profile_form($form, &$form_state, $account, Membersify_Membership $membership, Membersify_Profile $profile) {
  $form['question'] = array(
    '#markup' => t("Are you sure you want to delete the card %display?", array('%display' => $profile->display)),
  );
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Yes, delete this card'),
  );
  $form['actions']['cancel'] = array(
    '#markup' => l(t('Wait, I changed my mind'), 'user/' . $account->uid . '/membership/' . $membership->id . '/change_billing'),
  );

  return $form;
}

/**
 * Form validation handler for membersify_delete_profile_form().
 */
function membersify_delete_profile_form_validate($form, &$form_state) {
  try {
    $profile = $form_state['build_info']['args'][2];
    Membersify::setKeys(variable_get('membersify_public_key', ''), variable_get('membersify_secret_key', ''));
    $profile->delete();
  }
  catch (Exception $e) {
    form_set_error('submit', $e->getMessage());
  }
}

/**
 * Form submission handler for membersify_delete_profile_form().
 */
function membersify_delete_profile_form_submit($form, &$form_state) {
  drupal_set_message("The card has been deleted.");
  $account = $form_state['build_info']['args'][0];
  $membership = $form_state['build_info']['args'][1];
  $form_state['redirect'] = 'user/' . $account->uid . '/membership/' . $membership->id . '/change_billing';
}

/**
 * Menu callback: The view membership payments history page.
 *
 * @param stdClass $account
 *   The user account.
 * @param Membersify_Membership $membership
 *   The Membersify_Membership object.
 *
 * @see membersify_menu()
 */
function membersify_payment_history_page($account, Membersify_Membership $membership) {
  $build = array();

  $header = array(
    'date' => array(
      'data' => t('Date'),
      'field' => 'created',
      'sort' => 'desc',
    ),
    'type' => array(
      'data' => t('Type'),
      'field' => 'type',
    ),
    'txn_id' => array(
      'data' => t('Transaction ID'),
      'field' => 'txn_id',
    ),
    'amount' => array(
      'data' => t('Amount'),
      'field' => 'amount',
    ),
    'actions' => array(
      'data' => t('Actions'),
    ),
  );
  $rows = array();

  $query = db_select('membersify_history', 'h');
  $query->fields('h');
  $query->condition('h.membership_id', $membership->id);
  $query = $query->extend('TableSort')->orderByHeader($header);
  $query = $query->extend('PagerDefault')->limit(20);
  $result = $query->execute();

  foreach ($result as $row) {
    $history_item = new Membersify_HistoryItem();
    $history_item->setValues($row);
    $actions = array();
    $actions[] = l(t('View details'), 'user/' . $membership->user_id . '/membership/' . $membership->id . '/history/' . $history_item->id);

    $rows[] = array(
      format_date($history_item->created, 'small'),
      membersify_get_history_item_type($history_item->type),
      $history_item->txn_id,
      membersify_format_money($history_item->amount, $history_item->currency),
      implode(' | ', $actions),
    );
  }

  $build['payments_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#attributes' => array('id' => 'membersify-payments-history-table'),
    '#empty' => t('No history yet.')
  );
  $build['pager'] = array(
    '#theme' => 'pager',
    '#tags' => NULL,
  );

  return $build;
}

/**
 * Menu callback: The view membership payments history page.
 *
 * @param stdClass $account
 *   The user account.
 * @param Membersify_Membership $membership
 *   The Membersify_Membership object.
 * @param Membersify_HistoryItem $history_item
 *   The membershify history item to show the invoice for.
 *
 * @see membersify_menu()
 */
function membersify_payment_invoice_page($account, Membersify_Membership $membership, Membersify_HistoryItem $history_item) {
  $build = array();

  $build['invoice'] = array(
    '#theme' => 'membersify_invoice',
    '#account' => $account,
    '#membership' => $membership,
    '#history_item' => $history_item,
  );

  return $build;
}